package com.dukascopy.visualforex.schweizerkatz;

import java.util.*;
import com.dukascopy.api.*;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.concurrent.CopyOnWriteArrayList;
import java.lang.reflect.*;
import java.math.BigDecimal;

/* Developed by schweizerkatz, no warranty
 *
 * Created by VisualJForex Generator, version 1.76
 * Date: 08.07.2015 15:28
 */
public class oneshot3 implements IStrategy {

    private CopyOnWriteArrayList<TradeEventAction> tradeEventActions = new CopyOnWriteArrayList<TradeEventAction>();
    private static final String DATE_FORMAT_NOW = "yyyyMMdd_HHmmss";
    private IEngine engine;
    private IConsole console;
    private IHistory history;
    private IContext context;
    private IIndicators indicators;
    private IUserInterface userInterface;

    @Configurable("_entry:")
    public double _entry;
    @Configurable("_SL:")
    public double _SL;
    @Configurable("_profitMultiplier:")
    public double _profitMultiplier = 3.0;
    @Configurable("_MaxPipsOnAutoSize:")
    public int _MaxPipsOnAutoSize = 0;
    @Configurable("defaultTradeAmount:")
    public double defaultTradeAmount = 0.001;
    @Configurable("defaultInstrument:")
    public Instrument defaultInstrument = Instrument.EURUSD;

    private String AccountCurrency = "";
    private double _SL_range;
    private double Leverage;
    private int defaultSlippage = 1;
    private int defaultTakeProfit = 50;
    private Tick LastTick =  null ;
    private String AccountId = "";
    private double Equity;
    private double UseofLeverage;
    private double _TP;
    private List<IOrder> PendingPositions =  null ;
    private List<IOrder> AllPositions =  null ;
    private int OverWeekendEndLeverage;
    private int MarginCutLevel;
    private Candle LastAskCandle =  null ;
    private Period defaultPeriod = Period.ONE_HOUR;
    private boolean GlobalAccount;
    private int defaultStopLoss = 25;
    private List<IOrder> OpenPositions =  null ;
    private IMessage LastTradeEvent =  null ;
    private Candle LastBidCandle =  null ;


    public void onStart(IContext context) throws JFException {
        this.engine = context.getEngine();
        this.console = context.getConsole();
        this.history = context.getHistory();
        this.context = context;
        this.indicators = context.getIndicators();
        this.userInterface = context.getUserInterface();

        subscriptionInstrumentCheck(defaultInstrument);

        ITick lastITick = context.getHistory().getLastTick(defaultInstrument);
        LastTick = new Tick(lastITick, defaultInstrument);

        IBar bidBar = context.getHistory().getBar(defaultInstrument, defaultPeriod, OfferSide.BID, 1);
        IBar askBar = context.getHistory().getBar(defaultInstrument, defaultPeriod, OfferSide.ASK, 1);
        LastAskCandle = new Candle(askBar, defaultPeriod, defaultInstrument, OfferSide.ASK);
        LastBidCandle = new Candle(bidBar, defaultPeriod, defaultInstrument, OfferSide.BID);

        subscriptionInstrumentCheck(Instrument.fromString("EUR/USD"));
        
        If_block_71(0);  //<<----- added from ontick
    }

    public void onAccount(IAccount account) throws JFException {
        AccountCurrency = account.getCurrency().toString();
        Leverage = account.getLeverage();
        AccountId= account.getAccountId();
        Equity = account.getEquity();
        UseofLeverage = account.getUseOfLeverage();
        OverWeekendEndLeverage = account.getOverWeekEndLeverage();
        MarginCutLevel = account.getMarginCutLevel();
        GlobalAccount = account.isGlobal();
    }

    private void updateVariables(Instrument instrument) {
        try {
            AllPositions = engine.getOrders(instrument);
            List<IOrder> listMarket = new ArrayList<IOrder>();
            for (IOrder order: AllPositions) {
                if (order.getState().equals(IOrder.State.FILLED)){
                    listMarket.add(order);
                }
            }
            List<IOrder> listPending = new ArrayList<IOrder>();
            for (IOrder order: AllPositions) {
                if (order.getState().equals(IOrder.State.OPENED)){
                    listPending.add(order);
                }
            }
            OpenPositions = listMarket;
            PendingPositions = listPending;
        } catch(JFException e) {
            e.printStackTrace();
        }
    }

    public void onMessage(IMessage message) throws JFException {
        if (message.getOrder() != null) {
            updateVariables(message.getOrder().getInstrument());
            LastTradeEvent = message;
            for (TradeEventAction event :  tradeEventActions) {
                IOrder order = message.getOrder();
                if (order != null && event != null && message.getType().equals(event.getMessageType())&& order.getLabel().equals(event.getPositionLabel())) {
                    Method method;
                    try {
                        method = this.getClass().getDeclaredMethod(event.getNextBlockId(), Integer.class);
                        method.invoke(this, new Integer[] {event.getFlowId()});
                    } catch (SecurityException e) {
                            e.printStackTrace();
                    } catch (NoSuchMethodException e) {
                          e.printStackTrace();
                    } catch (IllegalArgumentException e) {
                        e.printStackTrace();
                    } catch (IllegalAccessException e) {
                        e.printStackTrace();
                    } catch (InvocationTargetException e) {
                        e.printStackTrace();
                    } 
                    tradeEventActions.remove(event); 
                }
            }   
        }
    }

    public void onStop() throws JFException {
    }

    public void onTick(Instrument instrument, ITick tick) throws JFException {
        LastTick = new Tick(tick, instrument);
        updateVariables(instrument);

      //      If_block_71(0);
    }

    public void onBar(Instrument instrument, Period period, IBar askBar, IBar bidBar) throws JFException {
        LastAskCandle = new Candle(askBar, period, instrument, OfferSide.ASK);
        LastBidCandle = new Candle(bidBar, period, instrument, OfferSide.BID);
        updateVariables(instrument);

    }

    public void subscriptionInstrumentCheck(Instrument instrument) {
        try {
              if (!context.getSubscribedInstruments().contains(instrument)) {
                  Set<Instrument> instruments = new HashSet<Instrument>();
                  instruments.add(instrument);
                  context.setSubscribedInstruments(instruments, true);
                  Thread.sleep(100);
              }
          } catch (InterruptedException e) {
              e.printStackTrace();
          }
        }

    public double round(double price, Instrument instrument) {
        BigDecimal big = new BigDecimal("" + price); 
        big = big.setScale(instrument.getPipScale() + 1, BigDecimal.ROUND_HALF_UP); 
        return big.doubleValue(); 
    }

    public ITick getLastTick(Instrument instrument) {
        try { 
            return (context.getHistory().getTick(instrument, 0)); 
        } catch (JFException e) { 
             e.printStackTrace();  
         } 
         return null; 
    }

    private  void If_block_71(Integer flow) {
        Instrument argument_1 = LastTick.getInstrument();
        Instrument argument_2 = defaultInstrument;
        if (argument_1!= null && !argument_1.equals(argument_2)) {
        }
        else if (argument_1!= null && argument_1.equals(argument_2)) {
            If_block_74(flow);
        }
    }

    private  void If_block_74(Integer flow) {
        double argument_1 = _entry;
        double argument_2 = _SL;
        if (argument_1< argument_2) {
            Calculation_block_82(flow);
        }
        else if (argument_1> argument_2) {
            Calculation_block_83(flow);
        }
        else if (argument_1== argument_2) {
        }
    }

    private  void PendingOpen_block_79(Integer flow) {
        double argument_1 = _entry;
        Instrument argument_2 = defaultInstrument;
        double argument_3 = defaultTradeAmount;
        int argument_4 = 0;
        double argument_5 = _SL;
        double argument_6 = _TP;
        String argument_7 = "";
        long argument_8 = 0;
        IEngine.OrderCommand command = IEngine.OrderCommand.SELLLIMIT;

        double stopLoss = round(argument_5, argument_2);
        double takeProfit = round(argument_6, argument_2);
        
        try {
            String label = getLabel();           
            IOrder order = context.getEngine().submitOrder(label, argument_2, command, argument_3, round(argument_1, argument_2), (double)argument_4, stopLoss, takeProfit, argument_8, argument_7);
                StopStrategy_block_81(flow);
        } catch (JFException e) {
            e.printStackTrace();
        }
    }

    private  void PendingOpen_block_80(Integer flow) {
        double argument_1 = _entry;
        Instrument argument_2 = defaultInstrument;
        double argument_3 = defaultTradeAmount;
        int argument_4 = 0;
        double argument_5 = _SL;
        double argument_6 = _TP;
        String argument_7 = "";
        long argument_8 = 0;
        IEngine.OrderCommand command = IEngine.OrderCommand.BUYLIMIT;

        double stopLoss = round(argument_5, argument_2);
        double takeProfit = round(argument_6, argument_2);
        
        try {
            String label = getLabel();           
            IOrder order = context.getEngine().submitOrder(label, argument_2, command, argument_3, round(argument_1, argument_2), (double)argument_4, stopLoss, takeProfit, argument_8, argument_7);
                StopStrategy_block_81(flow);
        } catch (JFException e) {
            e.printStackTrace();
        }
    }

    private  void StopStrategy_block_81(Integer flow) {
         context.stop();    
    }

    private void Calculation_block_82(Integer flow) {
        double argument_1 = _SL;
        double argument_2 = _entry;
        _SL_range = argument_1 - argument_2;
        CalculationExpression_block_95(flow);
    }

    private void Calculation_block_83(Integer flow) {
        double argument_1 = _entry;
        double argument_2 = _SL;
        _SL_range = argument_1 - argument_2;
        CalculationExpression_block_96(flow);
    }

    private  void If_block_89(Integer flow) {
        int argument_1 = _MaxPipsOnAutoSize;
        int argument_2 = 0;
        if (argument_1< argument_2) {
        }
        else if (argument_1> argument_2) {
            CalculationExpression_block_90(flow);
        }
        else if (argument_1== argument_2) {
        }
    }

    private void CalculationExpression_block_90(Integer flow) {
        defaultTradeAmount = (_MaxPipsOnAutoSize*defaultTradeAmount)/(_SL_range*Math.pow(10,defaultInstrument.getPipScale()));
        If_block_99(flow);
    }

    private void CalculationExpression_block_95(Integer flow) {
        _TP = _entry-(_SL_range*_profitMultiplier);
        MultipleAction_block_102(flow);
    }

    private void CalculationExpression_block_96(Integer flow) {
        _TP = _entry+(_SL_range*_profitMultiplier);
        MultipleAction_block_101(flow);
    }

    private  void If_block_99(Integer flow) {
        double argument_1 = defaultTradeAmount;
        double argument_2 = 0.001;
        if (argument_1< argument_2) {
            Assign_block_100(flow);
        }
        else if (argument_1> argument_2) {
        }
        else if (argument_1== argument_2) {
        }
    }

    private  void Assign_block_100(Integer flow) {
        double argument_1 = 0.001;
        defaultTradeAmount =  argument_1;
        }

    private  void MultipleAction_block_101(Integer flow) {
        If_block_89(flow);
        PendingOpen_block_80(flow);
    }

    private  void MultipleAction_block_102(Integer flow) {
        If_block_89(flow);
        PendingOpen_block_79(flow);
    }

class Candle  {

    IBar bar;
    Period period;
    Instrument instrument;
    OfferSide offerSide;

    public Candle(IBar bar, Period period, Instrument instrument, OfferSide offerSide) {
        this.bar = bar;
        this.period = period;
        this.instrument = instrument;
        this.offerSide = offerSide;
    }

    public Period getPeriod() {
        return period;
    }

    public void setPeriod(Period period) {
        this.period = period;
    }

    public Instrument getInstrument() {
        return instrument;
    }

    public void setInstrument(Instrument instrument) {
        this.instrument = instrument;
    }

    public OfferSide getOfferSide() {
        return offerSide;
    }

    public void setOfferSide(OfferSide offerSide) {
        this.offerSide = offerSide;
    }

    public IBar getBar() {
        return bar;
    }

    public void setBar(IBar bar) {
        this.bar = bar;
    }

    public long getTime() {
        return bar.getTime();
    }

    public double getOpen() {
        return bar.getOpen();
    }

    public double getClose() {
        return bar.getClose();
    }

    public double getLow() {
        return bar.getLow();
    }

    public double getHigh() {
        return bar.getHigh();
    }

    public double getVolume() {
        return bar.getVolume();
    }
}
class Tick {

    private ITick tick;
    private Instrument instrument;

    public Tick(ITick tick, Instrument instrument){
        this.instrument = instrument;
        this.tick = tick;
    }

    public Instrument getInstrument(){
       return  instrument;
    }

    public double getAsk(){
       return  tick.getAsk();
    }

    public double getBid(){
       return  tick.getBid();
    }

    public double getAskVolume(){
       return  tick.getAskVolume();
    }

    public double getBidVolume(){
        return tick.getBidVolume();
    }

   public long getTime(){
       return  tick.getTime();
    }

   public ITick getTick(){
       return  tick;
    }
}

public class AssertException extends RuntimeException {

    public AssertException(Object primary, Object compared) {
        super("Primary object : " + primary.toString() + " is different from " + compared.toString());
    }
}
    protected String getLabel() {
        String label;
        label = "IVF" + getCurrentTime(LastTick.getTime()) + generateRandom(10000) + generateRandom(10000);
        return label;
    }

    private String getCurrentTime(long time) {
        SimpleDateFormat sdf = new SimpleDateFormat(DATE_FORMAT_NOW);
        return sdf.format(time);
    }

    private static String generateRandom(int n) {
        int randomNumber = (int) (Math.random() * n);
        String answer = "" + randomNumber;
        if (answer.length() > 3) {
            answer = answer.substring(0, 4);
        }
        return answer;
    }

    class TradeEventAction {
        private IMessage.Type messageType;
        private String nextBlockId = "";
        private String positionLabel = "";
        private int flowId = 0;

        public IMessage.Type getMessageType() {
            return messageType;
        }

        public void setMessageType(IMessage.Type messageType) {
            this.messageType = messageType;
        }

        public String getNextBlockId() {
            return nextBlockId;
        }

        public void setNextBlockId(String nextBlockId) {
            this.nextBlockId = nextBlockId;
        }
        public String getPositionLabel() {
            return positionLabel;
       }

        public void setPositionLabel(String positionLabel) {
            this.positionLabel = positionLabel;
        }
        public int getFlowId() {
            return flowId;
        }
        public void setFlowId(int flowId) {
            this.flowId = flowId;
        }
    }
}